include <gio.h>
include <gki.h>
include <gset.h>
include "lpl.h"
include "lpx.h"

# LPL_CALC_COORDS -- Determine how much to expand the wcs and shrink the
# viewport for each curve.  It is necessary to do this before any actual
# plotting in order to calculate and issue the exact scaling factors.

procedure lpl_calc_coords (lpl, lpx, gd, ncurves)

pointer	lpl			# longplot structure
pointer	lpx			# per-curve ranges etc.
pointer	gd			# real graphics device descriptor
int	ncurves

int	offset, i, j, nmajor
bool	series, overtrace, stack
real	pytotal, pyeach
real	vxmin_all,vxmax_all, vymin_all,vymax_all

#debug+
int	debug
common /deb/ debug
#debug-
begin
	series = (LPL_MULTIPLE(lpl) == MULT_SERIES)
	overtrace = (LPL_MULTIPLE(lpl) == MULT_OVERTRACE)
	stack = (LPL_MULTIPLE(lpl) == MULT_STACK ||
	    LPL_MULTIPLE(lpl) == MULT_STACK_DY)
#debug+
if (debug == YES) {
call printf ("lpl_calc_coords:  series=%b, overtrace=%b, stack=%b\n")
call pargb (series)
call pargb (overtrace)
call pargb (stack)
}
#debug-

	# Round the wcs for each curve; viewports will be OK if not stacking.
	vxmin_all = 0.0
	vxmax_all = 1.0
	vymin_all = 0.0
	vymax_all = 1.0

	do i = 1, ncurves {

	    # Round the wcs if explicitly requested or if log/elog scaling.

	    if (LPL_XROUND(lpl) == YES || LPL_XLOGLIN(lpl) != LINEAR) {
		if (LPL_XSCALE_INDEF(lpl) == NO && LPL_MAJRX_INDEF == YES)
		    nmajor = -1
		else
		    nmajor = LPL_MAJRX(lpl)
		call lpl_wcs_extend (LPL_XLOGLIN(lpl), nmajor,
		    LPL_XSCALE(lpl), LPX_WXMIN(lpx,i-1), LPX_WXMAX(lpx,i-1))
#debug+
if (debug == YES) {
call eprintf ("                  after wcs extension, wxmin %f, wxmax %f\n")
call pargr (LPX_WXMIN(lpx,i-1))
call pargr (LPX_WXMAX(lpx,i-1))
}
	    }
	    if (LPL_YROUND(lpl) == YES || LPL_YLOGLIN(lpl) != LINEAR) {
		if (LPL_YSCALE_INDEF(lpl) == NO && LPL_MAJRY_INDEF == YES)
		    nmajor = -1
		else
		    nmajor = LPL_MAJRY(lpl)
		call lpl_wcs_extend (LPL_YLOGLIN(lpl), nmajor,
		    LPL_YSCALE(lpl), LPX_WYMIN(lpx,i-1), LPX_WYMAX(lpx,i-1))
#debug+
if (debug == YES) {
call eprintf ("                  after wcs extension, wymin %f, wymax %f\n")
call pargr (LPX_WYMIN(lpx,i-1))
call pargr (LPX_WYMAX(lpx,i-1))
}
	    }

	    # If plotting overtraced or stacked, make all x-axes the same.
	    if (!series)
		do j = 1, i-1 {
		    LPX_WXMIN(lpx,j-1) = LPX_WXMIN(lpx,i-1)
		    LPX_WXMAX(lpx,j-1) = LPX_WXMAX(lpx,i-1)
		}

	    # If overplotting, make all y-axes the same.
	    if (overtrace)
		do j = 1, i-1 {
		    LPX_WYMIN(lpx,j-1) = LPX_WYMIN(lpx,i-1)
		    LPX_WYMAX(lpx,j-1) = LPX_WYMAX(lpx,i-1)
		}

#debug+
if (debug == YES) {
call eprintf ("           final wxmin(lpx,%d)=%g,wxmax=%g, wymin=%g,wymax=%g\n")
call pargi (i-1)
call pargr (LPX_WXMIN(lpx,i-1))
call pargr (LPX_WXMAX(lpx,i-1))
call pargr (LPX_WYMIN(lpx,i-1))
call pargr (LPX_WYMAX(lpx,i-1))
}
#debug-
	    # Shrink the viewport as necessary for title lines and axis labels.
	    # Draw title and xlabel only if first plot or plotting in series.

	    if (i == 1 || series) {
		offset = Memi[LPL_TITLENDX(lpl) + i - 1] - 1
		call lpl_get_viewport (gd, lpl, lpx, i,
		    TITLE(LPL_TITLE(lpl),offset), XLABEL(LPL_XLABEL(lpl)),
		    YLABEL(LPL_YLABEL(lpl)))
	    } else
		call lpl_get_viewport (gd, lpl, lpx, i, "", "", 
		    YLABEL(LPL_YLABEL(lpl)))

	    vxmin_all = max (vxmin_all, LPX_VXMIN(lpx,i-1))
	    vxmax_all = min (vxmax_all, LPX_VXMAX(lpx,i-1))
	    vymin_all = max (vymin_all, LPX_VYMIN(lpx,i-1))
	    vymax_all = min (vymax_all, LPX_VYMAX(lpx,i-1))
	}

	# Assign the plotter ranges, world and kernel scales for each curve,
	# according to the relative placement of multiple plots.

	if (stack) {

	    # In stacked plotting, all x-params must give the same-length axis.
	    # The individual viewports will in general overlap intentionally,
	    # as the y-offset in LPL_MULTIPLE applies to the base of each plot.

	    if (LPL_YSCALE_INDEF(lpl) == NO) {
		pytotal = 0.0
		do i = 1, ncurves {
		    pytotal = pytotal + (LPX_WYMAX(lpx,i-1) - 
			LPX_WYMIN(lpx,i-1)) / LPL_YSCALE(lpl)
		    LPX_WYSCALE(lpx,i-1) = LPL_YSCALE(lpl)
		}
	    } else {
		pytotal = LPL_PYMAX(lpl) - LPL_PYMIN(lpl)
		pyeach = pytotal / float (ncurves)

		do i = 1, ncurves
		    LPX_WYSCALE(lpx,i-1) = (LPX_WYMAX(lpx,i-1) -
			LPX_WYMIN(lpx,i-1)) / pyeach
	    }

	    # Now have individual world y scales for each curve.  Calculate
	    # the actual plotter ranges and reset the viewports for each.
	    LPX_PYMIN(lpx,0) = LPL_PYMIN(lpl)
	    LPX_VYMIN(lpx,0) = vymin_all
	    do i = 1, ncurves - 1 {
		LPX_PYMAX(lpx,i-1) = LPX_PYMIN(lpx,i-1) + (LPX_WYMAX(lpx,i-1) -
		    LPX_WYMIN(lpx,i-1)) / LPX_WYSCALE(lpx,i-1)
		LPX_PYMIN(lpx,i) = LPX_PYMAX(lpx,i-1)
		LPX_VYMAX(lpx,i-1) = LPX_VYMIN(lpx,i-1) + ((LPX_PYMAX(lpx,i-1) -
		    LPX_PYMIN(lpx,i-1)) / pytotal) * (vymax_all - vymin_all)
		LPX_VYMIN(lpx,i) = LPX_VYMAX(lpx,i-1)
	    }

	    LPX_PYMAX(lpx,ncurves-1) = LPX_PYMIN(lpx,ncurves-1) +
		(LPX_WYMAX(lpx,ncurves-1) - LPX_WYMIN(lpx,ncurves-1)) / 
		LPX_WYSCALE(lpx,ncurves-1)
	    LPX_VYMAX(lpx,ncurves-1) = vymax_all

	    # Since by convention all stacked / overtraced plots have a common
	    # x-axis, simply make pxmin,pxmax = those for 1st curve.
	    if (LPL_XSCALE_INDEF(lpl) == NO) {
		do i = 1, ncurves {
		    LPX_WXSCALE(lpx,i-1) = LPL_XSCALE(lpl)
		    LPX_PXMIN(lpx,i-1) = LPL_PXMIN(lpl)
		    LPX_PXMAX(lpx,i-1) = LPL_PXMIN(lpl) + (LPX_WXMAX(lpx,i-1) -
			LPX_WXMIN(lpx,i-1)) / LPL_XSCALE(lpl)
		}
	    } else {
		do i = 1, ncurves {
		    LPX_WXSCALE(lpx,i-1) = (LPX_WXMAX(lpx,i-1) -
			LPX_WXMIN(lpx,i-1)) / (LPL_PXMAX(lpl) - LPL_PXMIN(lpl))
		    LPX_PXMIN(lpx,i-1) = LPL_PXMIN(lpl)
		    LPX_PXMAX(lpx,i-1) = LPL_PXMAX(lpl)
		}
	    }

	    do i = 1, ncurves {
		LPX_VXMIN(lpx,0) = vxmin_all
		LPX_VXMAX(lpx,0) = vxmax_all
	    }

	} else if (overtrace) {
	    
	    # Overplotting; all x params are equal and all y params are equal.
	    # First the world scales.

	    if (LPL_XSCALE_INDEF(lpl) == NO)
		do i = 1, ncurves
		    LPX_WXSCALE(lpx,i-1) = LPL_XSCALE(lpl)
	    else
		do i = 1, ncurves
		    LPX_WXSCALE(lpx,i-1) = (LPX_WXMAX(lpx,i-1) -
			LPX_WXMIN(lpx,i-1)) / (LPL_PXMAX(lpl) - LPL_PXMIN(lpl))

	    # World yscale.
	    if (LPL_YSCALE_INDEF(lpl) == NO)
		do i = 1, ncurves
		    LPX_WYSCALE(lpx,i-1) = LPL_YSCALE(lpl)
	    else
		do i = 1, ncurves
		    LPX_WYSCALE(lpx,i-1) = (LPX_WYMAX(lpx,i-1) -
			LPX_WYMIN(lpx,i-1)) / (LPL_PYMAX(lpl) - LPL_PYMIN(lpl))

	    # Plotter and viewport ranges.
	    do i = 1, ncurves {
		LPX_PXMIN(lpx,i-1) = LPL_PXMIN(lpl)
		LPX_PXMAX(lpx,i-1) = LPL_PXMAX(lpl)
		LPX_PYMIN(lpx,i-1) = LPL_PYMIN(lpl)
		LPX_PYMAX(lpx,i-1) = LPL_PYMAX(lpl)
		LPX_VXMIN(lpx,i-1) = vxmin_all
		LPX_VXMAX(lpx,i-1) = vxmax_all
		LPX_VYMIN(lpx,i-1) = vymin_all
		LPX_VYMAX(lpx,i-1) = vymax_all
	    }

	} else {

	    # Plotting in series; each curve can have everything different.

	    if (LPL_XSCALE_INDEF(lpl) == NO)
		do i = 1, ncurves
		    LPX_WXSCALE(lpx,i-1) = LPL_XSCALE(lpl)
	    else
		do i = 1, ncurves
		    LPX_WXSCALE(lpx,i-1) = (LPX_WXMAX(lpx,i-1) -
			LPX_WXMIN(lpx,i-1)) / (LPL_PXMAX(lpl) - LPL_PXMIN(lpl))

	    if (LPL_YSCALE_INDEF(lpl) == NO)
		do i = 1, ncurves
		    LPX_WYSCALE(lpx,i-1) = LPL_YSCALE(lpl)
	    else
		do i = 1, ncurves
		    LPX_WYSCALE(lpx,i-1) = (LPX_WYMAX(lpx,i-1) -
			LPX_WYMIN(lpx,i-1)) / (LPL_PYMAX(lpl) - LPL_PYMIN(lpl))

	    # Now have individual world scales for each curve.  Calculate the
	    # actual plotter ranges for each.  (Viewports are OK from ggview).
	    do i = 1, ncurves {
		LPX_PXMIN(lpx,i-1) = LPL_PXMIN(lpl)
		LPX_PXMAX(lpx,i-1) = LPX_PXMIN(lpx,i-1) + (LPX_WXMAX(lpx,i-1) -
		    LPX_WXMIN(lpx,i-1)) / LPX_WXSCALE(lpx,i-1)
		LPX_PYMIN(lpx,i-1) = LPL_PYMIN(lpl)
		LPX_PYMAX(lpx,i-1) = LPX_PYMIN(lpx,i-1) + (LPX_WYMAX(lpx,i-1) -
		    LPX_WYMIN(lpx,i-1)) / LPX_WYSCALE(lpx,i-1)
	    }

	}

	# Finalize the number of major ticks in x and y now that we
	# have the final world data ranges and scales for each curve.

	do i = 1, ncurves {
	    if (LPL_MAJRX_INDEF(lpl) == YES)
		LPX_MAJRX(lpx,i-1) = nint ((LPX_WXMAX(lpx,i-1) -
		    LPX_WXMIN(lpx,i-1)) / LPX_WXSCALE(lpx,i-1))
	    else
		LPX_MAJRX(lpx,i-1) = LPL_MAJRX(lpl)

	    if (LPL_MAJRY_INDEF(lpl) == YES)
		LPX_MAJRY(lpx,i-1) = nint ((LPX_WYMAX(lpx,i-1) -
		    LPX_WYMIN(lpx,i-1)) / LPX_WYSCALE(lpx,i-1))
	    else
		LPX_MAJRY(lpx,i-1) = LPL_MAJRY(lpl)
	}

	# Now compute the plotter scales (those to be sent in the metacode).
	# Gescape scale is device_units / GKI_unit.

	do i = 1, ncurves {

	    if (!IS_INDEF(LPX_WXSCALE(lpx,i-1)) && LPX_WXSCALE(lpx,i-1) != 0.0)
		LPX_PXSCALE(lpx,i-1) = (LPX_WXMAX(lpx,i-1) - 
		    LPX_WXMIN(lpx,i-1)) / (LPX_WXSCALE(lpx,i-1) * GKI_MAXNDC *
		    (LPX_VXMAX(lpx,i-1) - LPX_VXMIN(lpx,i-1)))
	    else
		LPX_PXSCALE(lpx,i-1) = INDEFR

	    if (!IS_INDEF(LPX_WYSCALE(lpx,i-1)) && LPX_WYSCALE(lpx,i-1) != 0.0)
		LPX_PYSCALE(lpx,i-1) = (LPX_WYMAX(lpx,i-1) -
		    LPX_WYMIN(lpx,i-1)) / (LPX_WYSCALE(lpx,i-1) * GKI_MAXNDC *
		    (LPX_VYMAX(lpx,i-1) - LPX_VYMIN(lpx,i-1)))
	    else
		LPX_PYSCALE(lpx,i-1) = INDEFR
#debug+
if (debug == YES) {
call eprintf ("		final viewport for curve %d: %f %f %f %f\n")
call pargi (i)
call pargr (LPX_VXMIN(lpx, i-1))
call pargr (LPX_VXMAX(lpx, i-1))
call pargr (LPX_VYMIN(lpx, i-1))
call pargr (LPX_VYMAX(lpx, i-1))
}
#debug-
	}
end
